/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1beta2

import (
	"context"
	"time"

	v1beta2 "github.com/longhorn/longhorn-manager/k8s/pkg/apis/longhorn/v1beta2"
	scheme "github.com/longhorn/longhorn-manager/k8s/pkg/client/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// BackupBackingImagesGetter has a method to return a BackupBackingImageInterface.
// A group's client should implement this interface.
type BackupBackingImagesGetter interface {
	BackupBackingImages(namespace string) BackupBackingImageInterface
}

// BackupBackingImageInterface has methods to work with BackupBackingImage resources.
type BackupBackingImageInterface interface {
	Create(ctx context.Context, backupBackingImage *v1beta2.BackupBackingImage, opts v1.CreateOptions) (*v1beta2.BackupBackingImage, error)
	Update(ctx context.Context, backupBackingImage *v1beta2.BackupBackingImage, opts v1.UpdateOptions) (*v1beta2.BackupBackingImage, error)
	UpdateStatus(ctx context.Context, backupBackingImage *v1beta2.BackupBackingImage, opts v1.UpdateOptions) (*v1beta2.BackupBackingImage, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1beta2.BackupBackingImage, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1beta2.BackupBackingImageList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1beta2.BackupBackingImage, err error)
	BackupBackingImageExpansion
}

// backupBackingImages implements BackupBackingImageInterface
type backupBackingImages struct {
	client rest.Interface
	ns     string
}

// newBackupBackingImages returns a BackupBackingImages
func newBackupBackingImages(c *LonghornV1beta2Client, namespace string) *backupBackingImages {
	return &backupBackingImages{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the backupBackingImage, and returns the corresponding backupBackingImage object, and an error if there is any.
func (c *backupBackingImages) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1beta2.BackupBackingImage, err error) {
	result = &v1beta2.BackupBackingImage{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("backupbackingimages").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of BackupBackingImages that match those selectors.
func (c *backupBackingImages) List(ctx context.Context, opts v1.ListOptions) (result *v1beta2.BackupBackingImageList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1beta2.BackupBackingImageList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("backupbackingimages").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested backupBackingImages.
func (c *backupBackingImages) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("backupbackingimages").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a backupBackingImage and creates it.  Returns the server's representation of the backupBackingImage, and an error, if there is any.
func (c *backupBackingImages) Create(ctx context.Context, backupBackingImage *v1beta2.BackupBackingImage, opts v1.CreateOptions) (result *v1beta2.BackupBackingImage, err error) {
	result = &v1beta2.BackupBackingImage{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("backupbackingimages").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(backupBackingImage).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a backupBackingImage and updates it. Returns the server's representation of the backupBackingImage, and an error, if there is any.
func (c *backupBackingImages) Update(ctx context.Context, backupBackingImage *v1beta2.BackupBackingImage, opts v1.UpdateOptions) (result *v1beta2.BackupBackingImage, err error) {
	result = &v1beta2.BackupBackingImage{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("backupbackingimages").
		Name(backupBackingImage.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(backupBackingImage).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *backupBackingImages) UpdateStatus(ctx context.Context, backupBackingImage *v1beta2.BackupBackingImage, opts v1.UpdateOptions) (result *v1beta2.BackupBackingImage, err error) {
	result = &v1beta2.BackupBackingImage{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("backupbackingimages").
		Name(backupBackingImage.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(backupBackingImage).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the backupBackingImage and deletes it. Returns an error if one occurs.
func (c *backupBackingImages) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("backupbackingimages").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *backupBackingImages) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("backupbackingimages").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched backupBackingImage.
func (c *backupBackingImages) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1beta2.BackupBackingImage, err error) {
	result = &v1beta2.BackupBackingImage{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("backupbackingimages").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
